/*******************************************************************************


     Timer para Ventilador (1 a 6 horas)

     Autor : Cludio Larios

     Data: 13/12/2015

     C/ pic 12f675

     Aplicao: Temporizar ventilador para desligamento automtico aps perodo
     programado pelo usurio entre 1 a 6 horas (visualizao em leds).
     
     Verso para compilador CCS C
     
     Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
     Uso didtico apenas.

*******************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, BROWNOUT
#use fast_io(a)
#zero_ram
#rom 0x2100 = {0,0}


//bytes

#byte OPTION_REG = 0X81
#byte GPIO   = 0x05
#byte TRISIO = 0x85
#byte TMR0   = 0x01
#byte ANSEL  = 0X9F
#byte CMCON  = 0X19
//bits
#bit  T0IE  = 0XB.5
#bit  GIE   = 0XB.7
#bit  PEIE  = 0XB.6
#bit  GPIE  = 0XB.3
#bit  GPIF  = 0XB.0
#bit  T0IF  = 0XB.2
#bit  IOC2  = 0X96.2

#bit  gate   = 0x5.5
#bit  tgate   = 0x85.5  //saida de disparo do triac
#bit  sw     = 0x05.3  //entrada da chave seleo/start/stop

#bit  GP4    = 0x05.4  //led
#bit  GP1    = 0x05.1  //led
#bit  GP0    = 0x05.0  //led
#bit  TGP4    = 0x85.4  //led
#bit  TGP1    = 0x85.1  //led
#bit  TGP0    = 0x85.0  //led

//escolha da frequencia de rede
#define f60hz  //descomente para 60 hz e comente para 50hz

//variavis globais

int8 cnt,seg,min,rhor;
/*------------------------------------------------------------------------------
             INTERRUPO POR MUDANA DE ESTADO DO PINO 5 (GPIO2)
------------------------------------------------------------------------------*/
#int_ra
void isr_CHANGE(){  //
  GPIO&=GPIO;
 #ifdef  f60hz
  if(++cnt==120)
 #else
  if(++cnt==100)
 #endif
 {
    cnt=0;
   if(++seg>59) {
    seg=0;
    if(++min>59){
     min=0;
     if(rhor){
     --rhor;

     }
    }
   }
  }
}





/*==============================================================================
         ROTINA PRINCIPAL/ INICIALIZAO DE PORTS E REGISTRADORES
==============================================================================*/


void main() {

   ANSEL=0;
   CMCON=7;
   TRISIO= 0b011111;//trisio
   GPIO=0b001000;//gpio
   GPIE=1;//liga interrupo por mudana no pino
   IOC2=1;//liga interrupo individual no pino 5 por mudana de estado
   PEIE=1;//liga interrupo de perifricos
   GPIF=0;//apaga flag
   GIE=1;//liga geral das interrupes
   rhor=read_eeprom(0); //recupera ajuste de horas anterior
   if(rhor>6)rhor=0;//limita
   min=read_eeprom(1); //recupera contagem de min anterior
   if(min>59)min=0;//limita

/*==============================================================================
                    LAO PRINCIPAL DE REPETIO
==============================================================================*/
   for(;;){

    if(!sw){
     if(++rhor>6) {rhor=0; }
     min=0; seg=0; cnt=0;
     while(!sw) delay_ms(20);

    }
     GIE=0;
    TRISIO= 0b011111;//trisio
    switch (rhor){

    case 0: {GIE=0; break;} //desliga interrupes
    case 1: {GIE=1; GP0=0;TGP0=0; GP4=1;TGP4=0; break; }//led 1 aceso
    case 2: {GIE=1; GP4=0;TGP4=0; GP0=1;TGP0=0; break; }//led 2 aceso
    case 3: {GIE=1; GP1=0;TGP1=0; GP4=1;TGP4=0; break; }//led 3 aceso
    case 4: {GIE=1; GP4=0;TGP4=0; GP1=1;TGP1=0; break; }//led 4 aceso
    case 5: {GIE=1; GP0=0;TGP0=0; GP1=1;TGP1=0; break; }//led 5 aceso
    case 6: {GIE=1; GP1=0;TGP1=0; GP0=1;TGP0=0; break; }//led 6 aceso

    }
    GIE=1;
    if(read_eeprom(1)!=min) write_eeprom(1,min);
    if(read_eeprom(0)!=rhor) write_eeprom(0,rhor);
    if(rhor){ gate=1;} else{gate=0; }//aciona saida
    delay_ms(20);

     }//for(;;)

}//main
