/*******************************************************************************


     Timer para Ventilador (1 a 6 horas)

     Autor : Cludio Larios

     Data: 13/12/2015

     C/ pic 12f675

     Aplicao: Temporizar ventilador para desligamento automtico aps perodo
     programado pelo usurio entre 1 a 6 horas (visualizao em leds).

     Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
     Verso para uso com compilador XC8
     Uso didtico apenas.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#define _XTAL_FREQ    4000000
#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin, I/O function on GP5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-Up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF      // GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)



//eprom
__EEPROM_DATA(0,0,0xff,0xff,0xff,0xff,0xff,0xff);//situao inicial da eeprom ps programao do pic



//ports
#define  gate   GPIO5
#define  tgate  TRISIO5  //saida de disparo do triac
#define  sw     GPIO3  //entrada da chave seleo/start/stop

#define  GP4    GPIO4  //led
#define  GP0    GPIO0  //led
#define  TGP4   TRISIO4  //led
#define  TGP1   TRISIO1  //led
#define  TGP0   TRISIO0  //led

//escolha da frequencia de rede
#define f60hz  //descomente para 60 hz e comente para 50hz

//variavis globais

unsigned char cnt,seg,min,rhor;
/*------------------------------------------------------------------------------
             INTERRUPO POR MUDANA DE ESTADO DO PINO 5 (GPIO2)
------------------------------------------------------------------------------*/

void interrupt isr_CHANGE(){  //
    if(!GPIF)return;
  GPIO&=GPIO;
  GPIF=0;
 #ifdef  f60hz
  if(++cnt==120)
 #else
  if(++cnt==100)
 #endif
 {
    cnt=0;
   if(++seg>59) {
    seg=0;
    if(++min>59){
     min=0;
     if(rhor){
     --rhor;

     }
    }
   }
  }
}





/*==============================================================================
         ROTINA PRINCIPAL/ INICIALIZAO DE PORTS E REGISTRADORES
==============================================================================*/


int main(int argc, char** argv) {

   ANSEL=0;
   CMCON=7;
   TRISIO= 0b011111;//trisio
   GPIO=0b001000;//gpio
   GPIE=1;//liga interrupo por mudana no pino
   IOC2=1;//liga interrupo individual no pino 5 por mudana de estado
   PEIE=1;//liga interrupo de perifricos
   GPIF=0;//apaga flag
   GIE=1;//liga geral das interrupes
   rhor=eeprom_read(0); //recupera ajuste de horas anterior
   if(rhor>6)rhor=0;//limita
   min=eeprom_read(1); //recupera contagem de min anterior
   if(min>59)min=0;//limita

/*==============================================================================
                    LAO PRINCIPAL DE REPETIO
==============================================================================*/
   for(;;){

    if(!sw){
     if(++rhor>6) {rhor=0; }
     min=0; seg=0; cnt=0;
     while(!sw) __delay_ms(20);

    }
     GIE=0;
    TRISIO= 0b011111;//trisio
    switch (rhor){

    case 0: {GIE=0; break;} //desliga interrupes
    case 1: {GIE=1; GP0=0;TGP0=0; GP4=1;TGP4=0; break; }//led 1 aceso
    case 2: {GIE=1; GP4=0;TGP4=0; GP0=1;TGP0=0; break; }//led 2 aceso
    case 3: {GIE=1; GP1=0;TGP1=0; GP4=1;TGP4=0; break; }//led 3 aceso
    case 4: {GIE=1; GP4=0;TGP4=0; GP1=1;TGP1=0; break; }//led 4 aceso
    case 5: {GIE=1; GP0=0;TGP0=0; GP1=1;TGP1=0; break; }//led 5 aceso
    case 6: {GIE=1; GP1=0;TGP1=0; GP0=1;TGP0=0; break; }//led 6 aceso

    }
    GIE=1;
    if(eeprom_read(1)!=min) eeprom_write(1,min); 
    if(eeprom_read(0)!=rhor) eeprom_write(0,rhor); 
    if(rhor){ gate=1;} else{gate=0; }//aciona saida
    __delay_ms(20);

     }//for(;;)

   return (EXIT_SUCCESS);
}//main
